// Date and time functions using a DS3231 RTC connected via I2C and Wire lib
// 
// Usage: Open serial Monitor
//      Command to set RTC time and update DriftInfo.DriftStart in the RTC battery back memory
//      Command format: TYYMMDDHHMMSS
//      Example: 2012 Oct 21 6:30am is T121021063000
//Time set command is from DriftCorrectedRTC written by Howard Lim Chong Han
//published at: https://limchonghan.wordpress.com/driftcorrectedds1307/

#include <Wire.h>
#include "RTClib.h"  //Adafruit library https://github.com/adafruit/RTClib

RTC_DS3231 rtc;

char daysOfTheWeek[7][12] = {
  "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

void setup () {
  Serial.begin(9600);
  delay(3000); // wait for console opening
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }

  if (rtc.lostPower()) {
    Serial.println("RTC lost power, lets set the time!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
}

void loop () {
  if(Serial.available()) processSerialCommand(); 
  DateTime now = rtc.now();   
  Serial.print(now.year(), DEC);
  Serial.print('/');
  Serial.print(now.month(), DEC);
  Serial.print('/');
  Serial.print(now.day(), DEC);
  Serial.print(" (");
  Serial.print(daysOfTheWeek[now.dayOfTheWeek()]);
  Serial.print(") ");
  Serial.print(now.hour(), DEC);
  Serial.print(':');
  Serial.print(now.minute(), DEC);
  Serial.print(':');
  Serial.print(now.second(), DEC);
  Serial.println();
  delay(3000);
}

void processSerialCommand() {
  uint16_t y;
  uint8_t m, d, hh, mm, ss; 
  char c = Serial.read(); 
  switch(c){
  case 'T':
    // Command to set RTC
    // Command format: TYYMMDDHHMMSS
    // Example: 2012 Oct 21 1:23pm is T121021132300
    delay(100); // Wait for all data to arrive
    if( Serial.available() >= 12 ){  // process only if all expected data is available
      // Parse incomming 12 ASCII charaters into y,m,d,hh,mm,ss
      // no error checking for numeric values in YYMDDHHMMSS fields, so be carefull!
      c = Serial.read();       y = c - '0';
      c = Serial.read();       y = 10*y; y += c-'0'; y += 2000;
      c = Serial.read();       m = c - '0'; 
      c = Serial.read();       m = 10*m; m += c-'0';
      c = Serial.read();       d = c - '0';
      c = Serial.read();       d = 10*d; d += c-'0';
      c = Serial.read();       hh = c - '0';
      c = Serial.read();       hh = 10*hh; hh += c-'0';
      c = Serial.read();       mm = c - '0';
      c = Serial.read();       mm = 10*mm; mm += c-'0';
      c = Serial.read();       ss = c - '0';
      c = Serial.read();       ss = 10*ss; ss += c-'0';
      rtc.adjust(DateTime(y, m, d, hh, mm, ss));
      Serial.print("RTC Set to: "); 
    }
    break;
  }
  while(Serial.available()) Serial.read();  // clear serial buffer
}

